from django.db.models import Q
import autocomplete_light
from jmb.fax import models
from jmb.organization.models import Contact

class RecipientAutocomplete(autocomplete_light.AutocompleteBase):
    search_fields = ('organization__name', 'last_name', 'first_name', 'fax')
    choices = Contact.objects.all()

    def choices_for_request(self):
        choice_html_format = u'<span class="div" data-value="%s">%s</span>'

        assert self.choices is not None, 'choices should be a queryset'
        assert self.search_fields, 'autocomplete.search_fields must be set'
        q = self.request.GET.get('q', '')

        def construct_search(field_name):
            if field_name.startswith('^'):
                return "%s__istartswith" % field_name[1:]
            elif field_name.startswith('@'):
                return "%s__search" % field_name[1:]
            else:
                return "%s__icontains" % field_name

        conditions = Q()
        if q:
            for search_field in self.search_fields:
                conditions |= Q(**{construct_search(search_field): q})

        return self.choices.filter(conditions)

    def choice_label(self, choice):
        return "%s - %s (%s)" % (choice.organization, choice.last_name, choice.fax or choice.organization.fax)

    def choice_value(self, choice):
        return unicode(choice.fax or choice.organization.fax)

autocomplete_light.register(RecipientAutocomplete, name='RecipientAutocomplete')
