# coding: utf-8
import re
import datetime

import autocomplete_light

from django.contrib import admin
from django import forms
from django.utils.translation import ugettext as _
from django.forms.util import ErrorList
from django.forms.models import BaseInlineFormSet
from django.conf import settings

from jmb.fax.models import Fax, FaxNumber, Cover, Device, Document

class FaxFileInlineFormSet(BaseInlineFormSet):

    def clean(self):

        for form in self.forms:
            cleaned_data = form.cleaned_data
            # verifico sia validita sia non vuoto
            if form.is_valid() and len(cleaned_data.keys()):
                # father object
                fax = cleaned_data['fax']

                content_type = cleaned_data['file'].content_type
                if hasattr(fax, 'device') and fax.device.kpn_url:
                    # verifico i file utilizzabili
                    if content_type not in settings.FAX_ACCEPTED_CONTENT_TYPES[fax.device.type]:
                        # mozilla passa un content_type application/unknown...
                        ext = os.path.splitext(cleaned_data['file'].name)[1].strip('.')
                        if not ext in settings.FAX_ACCEPTED_EXTENSIONS[fax.device.type]:
                            msg = "Not allowed filetype/extension: % - %s"
                            raise forms.ValidationError(msg % (content_type, ext))

class DocumentInline(admin.TabularInline):
    model = Document
    fields = ('file', 'ordering',)
    extra = 0
    formset = FaxFileInlineFormSet
    
class CoverInline(admin.StackedInline):
    model = Cover
    fieldsets = (
        (None, {'fields': (
            ('societa_to', 'societa_from'),
            ('to_person', 'from_person'),
            'pag', 'oggetto', 'note'
        )}),
    )



class FaxForm(forms.ModelForm):
    recipient_number = forms.CharField(
        max_length=100, required=True,
        widget = autocomplete_light.TextWidget('RecipientAutocomplete', )
    )

    class Meta:
        model = Fax
        widgets = {
        	'recipient_number': autocomplete_light.TextWidget('RecipientAutocomplete', ),
       	}
    inlines = (DocumentInline, CoverInline)

    def clean_recipient_number(self):
        value = self.cleaned_data['recipient_number']

        m = re.search('\((.*)\)', value)
        value = m and m.group(1) or value
        return re.sub('[-\. /]', '', value)
