"""
Parsing XML delle informazioni che arrivano da kpn
"""
from datetime import datetime

# import xml.etree.ElementTree as ET
# NOTA: imort di xml crea problemi: risulta in un import error che viene 
# mal propagato a meno che sia importato *prima* di 'uno'

# inotre lxml da in modo facile la mappatura fra namespace e tag
from lxml import objectify

from codici import RET_CODES
from jmb.fax import models

def parse_status_from_xml(xml):
    """
    Parse the xml from kpn to get the status notification info

    :param xml: an xml 
    """

    info = {}

    # uso objectify solo per trovare il namespace
    # in questo modo sono tranquillo che anche in futuro
    # dovrebbe restare valido il parsing
    obj_status = objectify.fromstring(xml)

    m1 = obj_status.nsmap['m1']
    m2 = obj_status.nsmap['m2']

    notif = [i for i in obj_status.iter('{%s}notif' % m2)][0]
    dest   = notif.find('{%s}dest' % m2)
    info['id'] = notif.find('{%s}msg-id' % m2).text
    info['subject'] = notif.find('{%s}subject' % m2).text
    # info['seq_id'] = dest.find('{%s}seq-id' % m1).text
    info['sender'] = dest.find('{%s}address' % m1).text
    status = dest.find('{%s}status' % m1).text
    info['status'] = 4 if status == 'OK' else 5
    info['status_message'] = "%s: %s" % (status, RET_CODES[status])
    info['pages'] = int(dest.find('{%s}pages' % m1).text)
    date_time_str = dest.find('{%s}time' % m1).text
    info['date_sent'] = datetime.strptime(date_time_str, '%Y-%m-%dT%H:%M:%SZ')

    info['sender_number'] = info['sender'] 
    info['recipient_number'] = ''

    return info

def update_status(xml):

    # f = open('/tmp/dbg.xml', 'w')
    # f.write(xml)
    # f.close()
    info = parse_status_from_xml(xml)
    fax = models.Fax.objects.get(job=info['id'])
    fax.status = info['status']
    fax.pages = info['pages']
    fax.status_message = info['status_message']
    fax.date_sent = info['date_sent']
    fax.save()



    
