# -*- coding: utf-8 -*-

import os
import time
import datetime
import random
import commands

from django.conf import settings
from django.db import models
from django.utils.translation import ugettext_lazy as _
from django.utils.translation import ugettext as ugt
from django.contrib.auth.models import User, Group, Permission
from django.core.files.storage import FileSystemStorage
from django.db.models import Q
from django.db import connection
from django.core.exceptions import PermissionDenied
from django.contrib.contenttypes.models import ContentType
from django.template.defaultfilters import slugify

from jmb.core.db.models import DateModel, UserModel, StatusModel, OrderedModel
from jmb.core.middleware.thread_local import get_current_user
#from jmb.core.db.models import Settings as JSettings

#########################
# FAX                   #
#########################
class FaxManager(models.Manager):
    def get_query_set(self):
        user = get_current_user()
        groups = user.groups.all()

        qs = super(FaxManager, self).get_query_set()

        # FIXME : Questo if serve per lo script che chiama hylafax.
        if not user or user.is_anonymous():
            return qs

        if user.has_perm("fax.list_all_fax"):
            return qs

        q = Q(is_public=True) | \
                  (Q(is_public=False) & Q(submitter=user)) | \
                  (Q(is_public=False) & Q(notify_group__in=user.groups.all()))

        if user.has_perm("fax.list_all_fax_not_assigned"):
            q1 = Q(is_public=False)
            q2 = Q(notify_group__isnull=True)
            q = q | (q1 & q2)

        list_lperms = FaxPermission.objects.filter(codename__startswith='list')
        # Controllo quali numeri di fax l'utente puo' visualizzare
        #
        for perm in list_lperms:
            if user.has_perm("fax.%s" % perm.codename):
                # Recupero, a partire dai numeri di telefono associati ai permessi,
                 #i device (le tty) che l'utente puo vedere.
                # Questo perche da hylafax viene passato il device ed e' quello che
                # noi andiamo a salvare nel modello dei fax.
                try:
                    allow_device = [ device.name for device in Device.objects.filter(numbers__number = perm.number) ]
                except Exception, e:
                    return qs.none()

                qdevice = Q(device__in = allow_device)

                # Puo' vedere tutti i fax di quel numero?
                if perm.codename.find("list_all_faxes_from") >= 0:
                    q = q | qdevice

                # Puo' vedere solo quelli non assegnati?
                elif perm.codename.find("list_all_faxes_not_assigned_from") >= 0:
                    q1 = (qdevice & Q(notify_group=None))
                    q = q | q1

        return qs.filter(q)

class Fax(DateModel, UserModel):
    status = models.IntegerField (
        max_length = 3,
        default = '3',
        choices = settings.FAX_STATUS_CODES,
        verbose_name = _("status"),
    )
    job = models.CharField (
        blank = True,
        null = True,
        max_length = 15,
        verbose_name = _("job"),
        help_text = _("job help text"),
    )
    keywords = models.CharField (
        blank = True,
        null = True,
        max_length = 100,
        verbose_name = _("keywords"),
    )
    pages = models.PositiveSmallIntegerField (
        blank = True,
        null = True,
        max_length = 3,
        verbose_name = _("pages"),
    )
    date_required = models.DateTimeField (
        blank = True,
        null = True,
        verbose_name = _("date required"),
        help_text = _("date required help text"),
    )
    date_sent = models.DateTimeField (
        blank = True,
        null = True,
        verbose_name = _("date sent"),
    )
    ## submitter/receiver
    submitter = models.ForeignKey (
        User,
        blank = True,
        null = True,
        verbose_name = _("submitter"),
    )
    ## group that can view the fax
    notify_group = models.ForeignKey (
        Group,
        blank = True,
        null = True,
        verbose_name = _("notify group"),
    )
    recipient = models.CharField (
        blank = True,
        null = True,
        max_length = 100,
        verbose_name = _("recipient"),
    )
    recipient_number = models.CharField (
        max_length = 16,
        verbose_name = _("recipient number"),
    )
    sender = models.CharField (
        blank = True,
        null = True,
        max_length = 100,
        verbose_name = _("sender"),
    )
    sender_number = models.CharField (
        blank = True,
        null = True,
        max_length = 16,
        verbose_name = _("sender number"),
    )
    duration = models.CharField (
        blank = True,
        null = True,
        max_length = 10,
        verbose_name = _("duration"),
    )
    device = models.ForeignKey (
        "Device",
        verbose_name = _("device"),
    )
    error = models.CharField (
        blank = True,
        null = True,
        max_length = 150,
        verbose_name = _("error"),
    )
    subject = models.CharField (
        max_length = 300,
        verbose_name = _("subject"),
    )
    read = models.IntegerField (
        null = True, blank = True,
        choices = ((0,_('Not Read')), (1,_('Read'))),
        default = 0,
        verbose_name = _("read"),
    )
    priority = models.PositiveSmallIntegerField (
        blank = True,
        null = True,
        default = 127,
        verbose_name = _("priority"),
        choices = settings.FAX_PRIORITIES
    )
    sent_by = models.CharField (
        max_length = 1,
        verbose_name = _("Sent By"),
        choices = settings.FAX_SENT_BY,
        default = 'W'
    )
    status_message = models.CharField(
        max_length=200, blank=True, null=True,
        verbose_name=_('Status message')
    )

    objects = FaxManager()

    is_public = models.BooleanField(
        default = False,
    )

    class Meta:
        verbose_name = _("Fax")
        verbose_name_plural = _("Fax")
        permissions = (
            ("view_fax"                       , "Can view fax"),
            ("download_all_report"            , "Can view all reports"),
            ("download_all_document"          , "Can download all documents"),
            ("send_all_fax"                   , "Can send all faxes"),
            ("list_all_fax"                   , "Can list all faxes"),
            ("list_all_fax_not_assigned"      , "Can list all faxes not assigned"),
            ("delete_all_fax_to_send"         , "Can Delete ALL FAX TO SEND"),
            ("delete_all_own_fax_to_send"     , "Can Delete ALL OWN FAX TO SEND"),
            ("delete_all_owngroup_fax_to_send", "Can Delete ALL OWN GROUP FAX TO SEND"),
            ("delete_all_own_fax"             , "Can Delete ALL OWN FAX"),
            ("delete_all_owngroup_fax"        , "Can Delete ALL OWN GROUP FAX"),
        )
        ordering = ('-date_sent',)

    def has_cover(self):
        "Test if fax has cover w/o touching the db"
        if hasattr(self, 'cover'):
            return True
        return False

    def send(self, test=False):
        """
        Spedisce il fax eventualmente creando una cover apposita
        
        :param test: boolean. Don't really send, debug purpose (kpn only)
        """
        from jmb.fax.codema import send_via_codema
        from jmb.fax.kpn import send_via_kpn

        if not self.recipient_number:
            return

        self.date_sent = datetime.datetime.now()

        if self.device.type == 'kpn':
            return send_via_kpn(self, test=test)
        elif self.device.type == 'codema':
            send_via_codema(self, fax_content_path, pdf_cover_path)
        elif self.device.type == 'hyla':
            send_via_hylafax(self, fax_content_path, ps_cover_path)

    def mark_as_read(self):
        self.read = 1


    def can_be_delete(self):
        from jmb.core.middleware.thread_local import get_current_user
        user = get_current_user()
        user_group = [ g.name for g in user.groups.all() ]
        list_perms = user.get_all_permissions()

        ## no one can delete faxes that have already flown away
        # that would erroneously give the ideay that the operation was ``cancelled``
        # Received, Sent, Failed
        if not self.status in (2,4,5):
            return False

        ## controlla permessi di delete "ALL" o "STAFF"
        if user.is_superuser or "fax.delete_fax" in list_perms:
            return True

        # Se il fax e' in stato TO SEND controllo altri tipi
        # di permessi
        if self.status == 8:
            if "fax.delete_all_fax_to_send" in list_perms:
                return True

            if "fax.delete_all_owngroup_fax_to_send" in list_perms and \
                self.notify_group and self.notify_group.name in user_group:
                return True

            if "fax.delete_all_own_fax_to_send" in list_perms and \
                self.submitter == user:
                return True

        # Controllo permessi di cancellazione dei proprio fax
        if "fax.delete_all_owngroup_fax" in list_perms and \
           self.notify_group and self.notify_group.name in user_group:
            return True
        if "fax.delete_all_own_fax" in list_perms and \
           self.submitter == user:
            return True

        list_lperms = FaxPermission.objects.filter(codename__startswith='delete')
        # Controllo quali numeri di fax l'utente puo' visualizzare
        # perm.model_id rappresenta il device
        #
        for perm in list_lperms:
            if user.has_perm("fax.%s" % perm.codename):
                # Se il device del fax non e' lo stesso del permesso salto a prescindere
                # tt i controlli
                try:
                    allow_device = [ device.name for device in Device.objects.filter(numbers__number = perm.number) ]
                except Exception, e:
                    return False

                if self.device in allow_device:
                    if perm.codename.find("delete_all_fax_from") >= 0:
                        return True
                    if perm.codename.find("delete_own_fax_from") >= 0:
                        if self.submitter == user:
                            return True
                    if perm.codename.find("delete_owngroup_fax_from") >= 0:
                        if self.notify_group and self.notify_group.name in user_group:
                            return True

        return False

    def save(self, *args, **kwargs):
        super(Fax, self).save(*args, **kwargs)

    def delete(self):
        if not self.can_be_delete():
            raise PermissionDenied
        super(Fax, self).delete()

    def __unicode__(self):
        return unicode(self.subject)

class Document(DateModel, UserModel, OrderedModel):
    file = models.FileField (
        upload_to = settings.FAX_PATH_DOCUMENTS,
        verbose_name = _("file"),
    )
    fax = models.ForeignKey (
        Fax,
        related_name = 'documents'
    )

    class Meta:
        verbose_name = _("Document")
        verbose_name_plural = _("Documents")
        ordering = ('ordering', 'date_create',)

class Cover(DateModel, UserModel):
    fax = models.OneToOneField (
        Fax,
        related_name = 'cover'
    )
    societa_to = models.CharField(
        blank = True, null = True,
        verbose_name = _("To"),
        max_length = 100
    )
    societa_from = models.CharField(
        blank = True, null = True,
        verbose_name = _("From"),
        max_length = 100
    )
    to_person = models.CharField(
        blank = True, null = True,
        verbose_name = _("To Person"),
        max_length = 100
    )
    from_person = models.CharField(
        blank = True, null = True,
        verbose_name = _("From Person"),
        max_length = 100
    )
    num_fax = models.CharField(
        blank = True, null = True,
        verbose_name = _("Fax number"),
        max_length = 30
    )
    pag = models.CharField(
        blank = True, null = True,
        verbose_name = _("Pag."),
        help_text = _("Cover Include"),
        max_length = 2
    )
    oggetto = models.CharField(
        blank = True, null = True,
        verbose_name = _("Object"),
        max_length = 100
    )
    date_fax = models.DateTimeField(
        blank = True, null = True,
        verbose_name = _("Date")
    )
    note = models.TextField(
        blank = True, null = True,
        verbose_name = _("Note")
    )

    class Meta:
        verbose_name = _("Cover")
        verbose_name_plural = _("Cover")
    
    def save_as_pdf(self):
        """Check if a cover is needed and create it in case. Return path name
        of pdf and ps cover as a tuple, (None, None) if no cover is required.
        """
        from jmb.core.views.generic.create_oo_file import create_file
        import tempfile
        from django.template.loader import get_template
        from django.template import TemplateDoesNotExist

        ###################################
        # CONTROLLO E CREAZIONE COVER
        ###################################
        # Se un campo della richiesta cover e' compilato aggiungo la cover al fax
        pdf_cover_name = "odt_file_%s_%s.pdf" % (time.time(), random.randint(0,99))
        ps_cover_name  = "ps_file_%s_%s.ps" % (time.time(), random.randint(0,99))

        pdf_cover_path  = os.path.join('/tmp', pdf_cover_name)
        ps_cover_path  = os.path.join('/tmp', ps_cover_name)

        extra_context = {
            'societa_to' : self.societa_to, 
            'societa_from' : self.societa_from,
            'to_person' : self.to_person, 
            'from_person' : self.from_person,
            'fax' : self.fax, 
            'pages' : self.pag, 
            'oggetto' : self.oggetto,
            'date_f' : self.date_create.strftime("%d/%m/%Y %H : %M"), 
            'note' : self.note,
        }

        str_oofile = create_file(None, file_name=pdf_cover_name, 
                                 file_model='admin/fax/fax/cover.odt',
                                 file_type="pdf", save_in="/tmp/", extra_context=extra_context)

        cmd = "pdf2ps -sPAPERSIZE=a4 %s %s" % (pdf_cover_path, ps_cover_path)
        status, output = commands.getstatusoutput(cmd)
        return pdf_cover_path, ps_cover_path

class Device(UserModel, DateModel, StatusModel):
    name = models.CharField(
        max_length = 100, verbose_name = _("Name"),
        unique = True
    )
    "must be unique"
    description = models.CharField(
        max_length = 200,blank = True, null = True,
        verbose_name = _("Description")
    )
    type = models.CharField(choices=settings.FAX_DEVICE_TYPES, max_length=12) 
    from_user = models.CharField(
        max_length = 50, blank=True, null=True,
        help_text=_("The email authorized to send by Codema")
    )
    kpn_password = models.CharField(
        max_length = 50, blank=True, null=True
    )
    kpn_url = models.CharField(
        max_length = 80, blank=True, null=True,
        help_text=_("""
            URL used to get asyncronous notification from KPN
            service (MUST: port 80, 8080, or 50000+)"""
        )
    )
    "Indispensabile per codema"
    pop_host = models.CharField(
        max_length = 50, blank=True, null=True,
        help_text=_('IP or hostname of host to poll via POP3')
    )
    pop_user = models.CharField(
        max_length = 50, blank=True, null=True,
    )
    pop_use_ssl = models.BooleanField(
        default=False
    )
    pop_password = models.CharField(
        max_length = 50, blank=True, null=True
    )
    smtp_host = models.CharField(
        max_length = 50, blank=True, null=True,
        help_text=_('IP or hostname of host to use as SMTP relay')
    )
    """
    Può essere lasciato bianco nel qual caso verrà usato il server codema. È la soluzione
    più veloce ma non lascia log. Può essere invece impostato anche ``localhost``
    """
    smtp_user = models.CharField(
        max_length = 50, blank=True, null=True
    )
    smtp_password = models.CharField(
        max_length = 50, blank=True, null=True
    )
    smtp_port = models.IntegerField(
        default=25, blank=True, null=True
    )
    smtp_use_ssl = models.BooleanField(
        default=False
    )

    def __unicode__(self):
        return "%s" % (self.name,)

class FaxNumberManager(models.Manager):
    def get_query_set(self):
        user = get_current_user()
        qs = super(FaxNumberManager, self).get_query_set()

        if user.is_superuser or user.has_perm("fax.send_all_fax"):
            return qs

        list_lperms = FaxPermission.objects.filter(codename__startswith='send')

        q = Q()

        # Controllo da quali numeri di fax l'utente puo' inviare
        for perm in list_lperms:
            if user.has_perm("fax.%s" % perm.codename):
                q = q | Q(number = perm.number)

        return qs.filter(q)

class FaxNumber(DateModel, UserModel, StatusModel, OrderedModel):
    """
    Un numero di fax. Ad un numero sono collegati una lista di persone che possono
    ricevere una notifica quando un :class:`Fax` viene spedito o inviato e delle regole che
    determinano se detta notifica deve avvenire o meno.
    """
    device = models.ManyToManyField(
        Device,
        related_name = "numbers",
        verbose_name = _("Device")
    )
    number = models.CharField(
        max_length = 30,
        verbose_name = _("Number"),
        db_column = "dnumber",
        unique = True
    )
    name = models.CharField(
        max_length = 100,
        verbose_name = _("Name"),
        blank = True, null = True
    )
    # comma separated list
    notify_mails = models.CharField(
        max_length = 200,
        blank = True, null = True,
        verbose_name = _("Notify Mails")
    )
    send_at_received_fax = models.BooleanField()
    send_at_sent_fax = models.BooleanField()
    objects = FaxNumberManager()


    def __unicode__(self):
        if self.name:
            return "%s - %s" % (self.number, self.name)
        return self.number


    class Meta:
        verbose_name = _("Fax Number")
        verbose_name_plural = _("Fax Numbers")
        ordering = ['ordering',]


    def save(self, *args, **kwargs):
        if not self.id:
            content_type = ContentType.objects.get_for_model(self)
            super(FaxNumber, self).save(*args, **kwargs)
            for perm in settings.FAX_DEFAULT_PERMS:
                permission = FaxPermission()
                permission.name = perm[1] % self.number
                permission.codename = perm[0] % slugify(self.number)
                permission.content_type = content_type
                permission.number = self.number
                permission.save()
        else:
            old_number = FaxNumber.objects.get(pk = self.pk).number
            super(FaxNumber, self).save(*args, **kwargs)
            if old_number != self.number:
                for perm in settings.FAX_DEFAULT_PERMS:
                    f = FaxPermission.objects.get(codename = perm[0] % slugify(old_number))
                    f.name = perm[1] % self.number
                    f.codename = perm[0] % slugify(self.number)
                    f.save()


    def delete(self, *args, **kwargs):
        for perm in settings.FAX_DEFAULT_PERMS:
            f = FaxPermission.objects.get(codename = perm[0] % slugify(self.number)).delete()
        super(FaxNumber, self).delete(*args, **kwargs)

    @classmethod
    def get_notifiers_in(cls, device=None, number=None):
        """Return a list of people that should be notified when a :class:`Fax` arrive
        on a certain device/number. At most one device or number can be passed.

        :param number: a fax number
        :param device: a fax device
        """
        # Recupero gli indirizzi mail a cui dovranno essere spedite le notifiche
        # Creo un set in modo da non avere indirizzi mail duplicati
        recipient_list = set()
        device_number = []
        if device:
            fax_number = cls.objects.filter(device__name = device)[0]
        if number:
            fax_number = cls.objects.get(number = number)

        if not (device or number):
            # va bene se esiste un solo FaxNumber
            numbers = cls.objects.all()
            if len(numbers) == 1:
                fax_numbers = numbers[0]
            else:
                assert device or number, "You must provide at least a number or a device"

        if fax_number.send_at_received_fax:
            return [m.strip() for m in fax_number.notify_mails.split(",")]
        return []

    @classmethod
    def get_notifiers_out(cls, device=None, number=None):
        """Return a list of people that should be notified when a fax arrive
        on a certain device/number. At most one device or number can be passed.

        :param number: a fax number
        :param device: a fax device
        """
        # Recupero gli indirizzi mail a cui dovranno essere spedite le notifiche
        # Creo un set in modo da non avere indirizzi mail duplicati
        recipient_list = set()
        device_number = []
        if device:
            fax_numbers = cls.objects.filter(device__name = device)
        if number:
            fax_numbers = cls.objects.filter(number = number)

        if fax_numbers:
            # it shuould be just one!
            fax_number = fax_numbers[0]
            
            if fax_number.send_at_sent_fax:
                return [m.strip() for m in fax_number.notify_mails.split(",")]
        return []


############################
# FAX PERMISSIONS          #
############################
class FaxPermission(Permission, UserModel, DateModel):
    number = models.CharField(
        max_length = 30,
        verbose_name = _("Number"),
        db_column = "dnumber",
    )

def send_via_hylafax(fax, content_path, ps_cover_name):
    """Invia il fax usando hylafax

    :param fax: l'istanza di :class:`Fax`
    :param ps_cover_name: boolean. If a cover is needed
    """
    # Converto, per sicurezza, il file pdf in un a4 verticale
    # Infatti, quando si scanerrizza dalla stampante capita che il pdf creato
    # sia verticale ma con la page size a 842x596, invece di 596x842, e questo
    # fa si che il fax non venga spedito
    #
    info = {
        'path'       : '/usr/bin/sendfax',
        'start_time' : 'now',
        'fax_number' : fax.recipient_number,
        'file_name'  : content_path, #fax.file.path, #'/tmp/%s' % (ps_tmp_file,),
        'priority'   : fax.priority or 127,
        'device'     : fax.device + "@127.0.0.1"
        }

    cmd = "sudo %(path)s -a %(start_time)s -P %(priority)s"

    # Aggiungo effettivamente la cover come cover se anche un file e'
    # stato aggiunto, altrimenti la cover verra' spedita come file
    if ps_cover_name and fax.file:
        cmd = cmd + " -C " + ps_cover_name
    else:
        cmd = cmd + " -n"

    cmd = cmd + " -s a4 -B 9600 -D -d %(fax_number)s -h %(device)s %(file_name)s"
    cmd = cmd % info

    status, output = commands.getstatusoutput(cmd)

    if status:
        raise Exception

    ## output from sendfax:
    ## request id is 879 (group id 879) for host localhost (1 file)
    fax.job = output.split()[3]
    fax.save()

